# External Interface to code
from fastapi_jwt_auth import AuthJWT
from fastapi_jwt_auth.exceptions import AuthJWTException
from .base import create_jwt_token_by_email
from .configuration import configure, get_setting, get_AuthJWT_instance
from .backend import account_backend
from .backend import magic_backend
from .models import User, AccountProfile
from .routers.profile_router import profile_router



import logging
import sys
from dataclasses import dataclass
from typing import ClassVar
from dataclasses_json import DataClassJsonMixin
from pyvise.configuration import get_setting


logging.basicConfig(stream=sys.stdout, level=logging.INFO)


def get_logger(name):
    return logging.getLogger(name)


@dataclass
class HasuraBase(DataClassJsonMixin):
    """
    # [Reference Library](https://pypi.org/project/dataclasses-json/)
    # [GraphQL Queries](https://graphql.org/learn/queries/)
    """
    GRAPHQL_BASE: ClassVar[str] = get_setting('HASURA_ENDPOINT')
    __tablename__: ClassVar[str] = 'db_table_name'
    logger = get_logger(__name__)

    @classmethod
    def generate_fields(cls):
      instance = cls()
      field_strings = ""
      attributes = []

      for field in instance.__annotations__:
        if (not field.startswith('__')) and (field[0].islower()) :
          attributes.append(field)

      for idx, annotation in enumerate(attributes, start=1):        
        field_strings += annotation + ( ", " if idx < len(attributes) else "")

      return field_strings

    @classmethod
    def generate_singular_name(cls):
        return 'AccountProfile'



from magic_admin import Magic
from pyvise.base import get_logger, get_setting
from pyvise.models import User

logger = get_logger(__name__)


def get_magic_instance():
    magic = Magic(api_secret_key=get_setting('MAGIC_SECRET_KEY'),
                  retries=5, timeout=5, backoff_factor=0.01,
                  )
    return magic


def find_magic_user(email_address: str, did_token: str) -> User:
    try:
        magic = get_magic_instance()

        magic.Token.validate(did_token)
        issuer = magic.Token.get_issuer(did_token)
        magic_user_meta = magic.User.get_metadata_by_issuer(issuer)

        if(magic_user_meta.data.get('email') == email_address):
            # magic.User.get_metadata_by_public_address
            # magic.User.get_metadata_by_token
            return User(key=issuer, email_address=email_address)
        else:
            return None
    except Exception as exc:
        print(f"Exception thrown, suppressing and returning False {exc}")
        # logger.exception("Could not validate did token", exc)
    return None



from pyvise import graphy  # considering moving to own module
from pyvise.models import AccountProfile


def find_account_profile_by_email(email_address: str) -> AccountProfile:
    query_variables = {
        'email_address': email_address
    }
    graph_query = graphy.GraphQLQuery(
        query=AccountProfile.generate_find_by_query(email=email_address))
    client = graphy.GraphQLAPIClient()
    data = client.execute_single_query(graph_query, query_variables)
    account = data.get('account_profiles')
    if(len(account) > 0):
        account = AccountProfile.from_dict(account[0])
        return account
    else:
        return None


def create_account_profile(key: str, email_address: str, access_token: str, expiration: str):
    query = AccountProfile.generate_upsert_mutation()
    expiration_string = '2022-01-22'  # expiration
    query_variables = {
        'key': key,
        'email_address': email_address,
        'access_token': access_token,
        'expiration': expiration_string
    }
    graph_query = graphy.GraphQLQuery(query=query)
    client = graphy.GraphQLAPIClient()
    data = client.execute_single_query(graph_query, query_variables)
    # TODO: figure out return types in graphql
    account = data.get('insert_account_profiles_one')
    return account



from dataclasses import dataclass
from typing import ClassVar
from pyvise.graphy import HasuraBase


@dataclass
class User:
    """
    A domain user object.  This models individual users in the system.
    This is currently not stored in a database, but can be linked
    in the account_profile via email address today
    """
    key: str = None
    email_address: str = None
    friendly_name: str = None


@dataclass
class AccountProfile(HasuraBase):
    """
    GraphQL domain representation of profile Object
    """
    __tablename__: ClassVar[str] = 'account_profiles'
    __schema__: ClassVar[dict] = {
        'email_address': {'required': 'true'}
    }
    __operations__: ClassVar[dict] = {
        'create': 'insert_account_profiles_one'
    }

    key: str = None
    friendly_name: str = None
    email_address: str = None
    created_at: str = None
    current_role: str = None

    @classmethod
    def generate_upsert_mutation(cls):
        return """mutation CreateAccountProfile(
        $key: String, 
        $email_address: String, 
        $access_token: String,
        $expiration: timestamptz) {
            insert_account_profiles_one(
            object: {
                email_address: $email_address, 
                key: $key
                account_access_tokens:{
                data: {
                    access_token: $access_token
                    expiration: $expiration
                }
                }
            },
            on_conflict: {
                constraint: person_profile_pkey,
                update_columns: [email_address]
    }
            ) 
        {
            email_address
            key    
        }
        }
      """

    @classmethod
    def generate_find_by_query(cls, **kwargs):
        """
        TODO: parse kwargs to create the input parameter and equal value
        """
        return """
      query FindAccountProfileByEmailAddress($email_address: String) {
  account_profiles(where: {email_address: {_eq: $email_address}}, limit: 1) {
    email_address
    friendly_name
    key
    current_role
    updated_at
    created_at
  }
}
      """
